/* -*- C++ -*-
 * File: libraw_io.h
 * Copyright 2008 Alex Tutubalin <lexa@lexa.ru>
 * Created: Sat Mar  8 , 2008
 *
 * LibRaw internal I/O layer (interface)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef IODEFINES_H
#define IODEFINES_H

#ifdef WIN32
typedef long off_t;
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
{
    size_t pos;
    size_t size;
    size_t mmap_size;
    unsigned char *buf;
    void *chain;
} t_datastream;

DllDef int datastream_pushfile(const char *fname, t_datastream *stream);

DllDef int datastream_pop(t_datastream *stream);
DllDef int datastream_pushbytes(void *data, size_t size, t_datastream *stream);
DllDef int datastream_fopen(const char *fname, t_datastream *stream);
DllDef int datasteam_allocopen(size_t size, t_datastream *stream);
DllDef void datastream_closeall(t_datastream *stream);
DllDef void datastream_close(t_datastream *stream);
DllDef int datastream_getc(t_datastream *stream);
DllDef size_t datastream_fread(void *buf, size_t size, size_t n, t_datastream *stream);
DllDef int datastream_end(t_datastream *stream);
DllDef off_t datastream_tell(t_datastream *stream);
DllDef int datastream_seek(off_t pos, int whence, t_datastream *stream);
DllDef int datastream_scanf(t_datastream *stream, const char * fmt, void *ptr);
DllDef unsigned char * datastream_gets(unsigned char *buf, int sz,t_datastream *stream);

#ifndef IODEF_IMPL

#ifndef NO_OWN_IO

#define fgetc(stream) datastream_getc(stream)
#ifdef getc
#undef getc
#endif
#define getc(stream) datastream_getc(stream)

#define fread(b,s,n,f) datastream_fread(b,s,n,f)
#ifdef feof
#undef feof
#endif
#define feof(f) datastream_end(f)

#define ftello(f) datastream_tell(f)
#define ftell(f) datastream_tell(f)

#define fseek(f,off,w) datastream_seek(off,w,f)
#define fseeko(f,off,w) datastream_seek(off,w,f)

#define fscanf(f,ff,ptr) datastream_scanf(f,ff,ptr)
#define fgets(buf,size,ptr) datastream_gets((unsigned char*)buf,size,ptr)
#endif
#endif

#ifdef __cplusplus
};
#endif


#endif
